package org.ngikm;

import java.util.Optional;
import java.util.function.Consumer;
import java.util.function.Function;

import org.expeditee.gio.EcosystemManager;
import org.expeditee.gui.Frame;
import org.expeditee.items.Text;

public class Util {
    
	@FunctionalInterface
	public interface ThrowingConsumer<T, E extends Exception> extends Consumer<T> {
		@Override
		default void accept(final T t) {
			try {
				acceptThrows(t);
			} catch (final Exception e) {
				throw new RuntimeException(e);
			}
		}
		void acceptThrows(final T t) throws E;
	}
	
	@FunctionalInterface
	public interface ThrowingFunction<T, R, E extends Exception> extends Function<T, R> {
		@Override
		default R apply(T t) {
			try {
				return applyThrows(t);
			} catch (final Exception e) {
				throw new RuntimeException(e);
			}
		}
		R applyThrows(final T t) throws E;
	}
	
	public static Text AddText(final Frame toFrame, final int x, final int y, final String content) {
		final int windowHeight = EcosystemManager.getGraphicsManager().getWindowSize().height;
		final Optional<Text> item = toFrame.getTextItems().stream().filter(i -> { 
			final org.expeditee.core.Point p = i.getPosition(); 
			return (p.getX() == x) && (p.getY() == y); 
		}).findAny();
		if (item.isPresent()) {
			if (y >= (windowHeight * 0.9)) {
				return AddText(toFrame, x + 100, y, content);
			}
			return AddText(toFrame, x, y + 100, content);
		} else {
			final Text text = toFrame.addText(x, y, content, null, null);
			text.setOwner(toFrame.getOwner());
			return text;
		}
	}
}
